@@ -49,7 +49,7 @@ typedef enum {
         GDA_QUERY_TYPE_NON_PARSED_SQL
 } GdaQueryType;
 
-enum
+typedef enum
 {
 	GDA_QUERY_XML_LOAD_ERROR,
 	GDA_QUERY_META_DATA_UPDATE,
@@ -61,8 +61,9 @@ enum
 	GDA_QUERY_STRUCTURE_ERROR,
 	GDA_QUERY_SQL_ANALYSE_ERROR,
 	GDA_QUERY_NO_CNC_ERROR,
-	GDA_QUERY_CNC_CLOSED_ERROR
-};
+	GDA_QUERY_CNC_CLOSED_ERROR,
+	GDA_QUERY_EXEC_ERROR
+} GdaQueryError;
 
 
 /* struct for the object's data */
@@ -95,9 +96,9 @@ struct _GdaQueryClass
 };
 
 GType              gda_query_get_type               (void);
-GObject           *gda_query_new                    (GdaDict *dict);
-GObject           *gda_query_new_copy               (GdaQuery *orig, GHashTable *replacements);
-GObject           *gda_query_new_from_sql           (GdaDict *dict, const gchar *sql, GError **error);
+GdaQuery          *gda_query_new                    (GdaDict *dict);
+GdaQuery          *gda_query_new_copy               (GdaQuery *orig, GHashTable *replacements);
+GdaQuery          *gda_query_new_from_sql           (GdaDict *dict, const gchar *sql, GError **error);
 
 void               gda_query_set_query_type         (GdaQuery *query, GdaQueryType type);
 GdaQueryType       gda_query_get_query_type         (GdaQuery *query);
@@ -107,12 +108,12 @@ gboolean           gda_query_is_insert_query        (GdaQuery *query);
 gboolean           gda_query_is_update_query        (GdaQuery *query);
 gboolean           gda_query_is_delete_query        (GdaQuery *query);
 gboolean           gda_query_is_modif_query         (GdaQuery *query);
+gboolean           gda_query_is_well_formed         (GdaQuery *query, GdaParameterList *context, GError **error);
 GdaQuery          *gda_query_get_parent_query       (GdaQuery *query);
 
 GSList            *gda_query_get_parameters         (GdaQuery *query);
-GdaParameterList  *gda_query_get_parameters_boxed   (GdaQuery *query);
-#define            GDA_QUERY_EXEC_FAILED ((void *) -1)
-GdaDataModel      *gda_query_execute                (GdaQuery *query, GdaParameterList *params,
+GdaParameterList  *gda_query_get_parameter_list   (GdaQuery *query);
+GdaObject         *gda_query_execute                (GdaQuery *query, GdaParameterList *params,
 						     gboolean iter_model_only_requested, GError **error);
 
 
@@ -154,8 +155,10 @@ GdaQueryField     *gda_query_add_field_from_sql     (GdaQuery *query, const gcha
 
 GSList            *gda_query_get_all_fields         (GdaQuery *query);
 GdaQueryField     *gda_query_get_field_by_sql_naming        (GdaQuery *query, const gchar *sql_naming);
-GdaQueryField     *gda_query_get_field_by_ref_field         (GdaQuery *query, GdaQueryTarget *target, GdaEntityField *ref_field, 
-								 GdaQueryFieldState field_state);
+GdaQueryField     *gda_query_get_field_by_param_name        (GdaQuery *query, const gchar *param_name);
+GdaQueryField     *gda_query_get_field_by_ref_field         (GdaQuery *query, GdaQueryTarget *target, 
+							     GdaEntityField *ref_field, 
+							     GdaQueryFieldState field_state);
 GdaQueryField     *gda_query_get_first_field_for_target     (GdaQuery *query, GdaQueryTarget *target);
 GSList            *gda_query_expand_all_field               (GdaQuery *query, GdaQueryTarget *target);
 void               gda_query_order_fields_using_join_conds  (GdaQuery *query);
